# -*- coding: utf-8 -*-

"""
Matchering Log
~~~~~~~~~~~~~~~~~~~~~

Internal Matchering logging module that can be bound to any text stream.

:copyright: (C) 2016-2020 Sergree
:license: GPLv3, see LICENSE for more details.

"""

from .codes import Code
from .handlers import warning, info, debug, debug_line
from .exceptions import ModuleError
