# -*- coding: utf-8 -*-

"""
Matchering Hyrax Limiter
~~~~~~~~~~~~~~~~~~~~~

Internal Matchering brickwall limiter based on the use of time constant and Butterworth filters with sliding Windows.

:copyright: (C) 2016-2020 Sergree
:license: GPLv3, see LICENSE for more details.

"""

from .hyrax import limit
